
/*
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/* Scheduler include files. */
#include "FreeRTOS.h"
#include "task.h"

#include <avr/interrupt.h>

/* Pololu include files. */
// Don't forget that the Pololu Libraries need to be be recompiled to avoid the LCD using Pin 10.
#include <pololu/orangutan.h>

#include <ServoPWMTimer.h>


void start_PWM_hardware()
{
    taskENTER_CRITICAL();
        
#if defined( portUSE_TIMER1_PWM )

    pwmTCCRb = pwmFAST_NONINVERTED_b | pwmPRESCALE_8; 

    /* pwmIRC1 = configCPU_CLOCK_HZ/(pwmCLOCK_PRESCALER*pwmBASE_HZ) - 1 = 55,295 with pwmPRESCALE_8 */
    // This calculation result applies to 22118400Hz. 
    pwmICR1 = configCPU_CLOCK_HZ/(pwmCLOCK_PRESCALER*pwmBASE_HZ) - 1; // 50Hz or 20mS cycles.
                                                 
    DDRB |= pwmDDRB; // PB1 OC1A (Pin 9) & PB2 OC1B (Pin 10)
    
    #warning "Timer1 used for PWM."
                                                    
#elif defined( portUSE_TIMER0_PWM )
    
    // OutputHZ = configCPU_CLOCK_HZ/(pwmCLOCK_PRESCALER*510) = 42.352Hz with pwmPRESCALE_1024 Phase Correct PWM
    // This calculation result applies to 22118400Hz. 
    pwmTCCRb = pwmPRESCALE_1024;                            // always counts to 0xFF
   
    DDRD |= pwmDDRD; // PD6 OC0A (Pin 6) & PD5 OC0B (Pin 5)
    
    #warning "Timer0 used for PWM."
   
#endif

    // set servo A to mid range, being 3/40 or 1500uS/20000uS
    pwmOCRa = (unsigned int)( (configCPU_CLOCK_HZ/(pwmCLOCK_PRESCALER*pwmBASE_HZ) - 1) / 5 * 3 / 8 ); 

     // set servo B to mid range, being 3/40 or 1500uS/20000uS
    pwmOCRb = (unsigned int)( (configCPU_CLOCK_HZ/(pwmCLOCK_PRESCALER*pwmBASE_HZ) - 1) / 5 * 3 / 8 );

    pwmTCCRa = pwmFAST_NONINVERTED_a; // turn on the PWM...
    
    taskEXIT_CRITICAL();
}

void stop_PWM_hardware()
{
    taskENTER_CRITICAL();
    
    pwmTCCRa = 0;
    
#if defined( portUSE_TIMER1_PWM )

    DDRB &= ~pwmDDRB; // PB1 OC1A (Pin 9) & PB2 OC1B (Pin 10)
                                        
#elif defined( portUSE_TIMER0_PWM )
   
    DDRD &= ~pwmDDRD; // PD6 OC0A (Pin 6) & PD5 OC0B (Pin 5)
   
#endif
    
    taskEXIT_CRITICAL();   
}


void set_PWM_hardware( unsigned int servoA, unsigned int servoB)     // PWM pulse width in uS
{
/*-----------------------------------------------------------*/ 

    /*  This is testing code cruft:
        It doesn't implement the proper semaphore process to get exclusive access to the LCD.
        But it will go once the clock is finished.
    */
     
    lcd_goto_xy(5, 2);              // go to the sixth character of the third LCD line
    print_long( servoA);
    print("  ");
    lcd_goto_xy(11, 2);              // go to the twelveth character of the third LCD line
    print_long( servoB);
    print("  ");			        // added spaces are to overwrite left over chars   


    /*  This is testing code cruft:
    set_digital_output( IO_D3, 0);               // red LED on to show process
    vTaskDelay( 500 / portTICK_RATE_MS );
    set_digital_output( IO_D3, 1);               // red LED on to show process 
    vTaskDelay( 1000 / portTICK_RATE_MS );
    set_digital_output( IO_D3, 0);               // red LED on to show process
    vTaskDelay( 500 / portTICK_RATE_MS );
    set_digital_output( IO_D3, 1);               // red LED on to show process  
    vTaskDelay( 500 / portTICK_RATE_MS );
    set_digital_output( IO_D3, 0);               // red LED on to show process */
            
    /*-----------------------------------------------------------*/ 
    
    taskENTER_CRITICAL();
    
    
#if defined( portUSE_TIMER1_PWM )

    // pwmOCRx is set in uS, so convert from uS to # of steps to pwmICR1
    pwmOCRa = (unsigned int)( (float)servoA * (configCPU_CLOCK_HZ/(pwmCLOCK_PRESCALER*pwmBASE_HZ) - 1) * pwmBASE_HZ / 1000000.0 );

    pwmOCRb = (unsigned int)( (float)servoB * (configCPU_CLOCK_HZ/(pwmCLOCK_PRESCALER*pwmBASE_HZ) - 1) * pwmBASE_HZ / 1000000.0 );
                                                      
#elif defined( portUSE_TIMER0_PWM )
   
    pwmOCRa = (char)( (float)servoA * configCPU_CLOCK_HZ  / pwmCLOCK_PRESCALER / 1000000 ); // This calculation is just wrong,

    pwmOCRb = (char)( (float)servoB * configCPU_CLOCK_HZ  / pwmCLOCK_PRESCALER / 1000000 ); // but I have no interest to fix it.
    
#endif
    
    taskEXIT_CRITICAL();   

}



