#ifndef CHEADER_RETROGRADE
# define CHEADER_RETROGRADE

/*
File:   retrograde.h
*/

/* Define the addresses of the devices that we want to manage on i2c bus.*/
#define DS1307          0xD0      // device address of DS1307 Real Time Clock is 0xD0, see datasheet
#define TPA81           0xD0      // device address of TPA81 Thermopile is 0xD0, see datasheet
#define SRF10           0xE0      // device address of SRF10 Sonar is 0xE0, see datasheet

/* definitions for use in the Control byte */
# define SQWDISABLEOUT  10000000b   //  level SQW should take if DISABLED
# define SQWENABLE      00010000b   //  set to 1 to enable SQW output
# define SQWRS1         00000010b   //  RS1 & RS0 set to 0 for 1Hz output
# define SQWRS0         00000001b


/* structure to pass the Analogue Sensor parameters */
typedef struct
{
    unsigned int   adc0;        // ADC value from sensor 0
//    unsigned int   adc1;        // ADC value from sensor 1
//    unsigned int   adc2;        // ADC value from sensor 2
//    int           TempC;      // Calculated value to pass if desired
} xADCArray, * pADCArray;


/*
 * Declare a variable of type xQueueHandle.
 * this queue will have the ADC value written to it.
 */
xQueueHandle xADCQueue;

/*
 * Declare a variable of type xQueueHandle.
 * One queue will have the i2c RTC values written to it.
 */
xQueueHandle xI2CRTCQueue;


/* Create a Semaphore mutex flag for the ADC. To ensure only single access. */
xSemaphoreHandle xADCSemaphore;

/* Create a Semaphore mutex flag for the i2c Bus. To ensure only single access. */
xSemaphoreHandle xI2CSemaphore;

/* Create a Semaphore mutex flag for the LCD. To ensure only single access. */
xSemaphoreHandle xLCDSemaphore;

/* Create a Semaphore mutex flag to trigger Retrograde Analogue Hands. */
xSemaphoreHandle xRetrogradeSemaphore;


/*-----------------------------------------------------------*/


static void TaskBlinkGreenLED(void *pvParameters); // Main Arduino (Green) LED Blink

static void TaskReadADCSensors(void *pvParameters);       // Read ADC Sensor Values (Sharp IR)

static void TaskReadI2CRTC(void *pvParameters);   // Read I2C Bus for RTC

static void TaskWriteAnalogLCD(void *pvParameters); // Write ADC Sensors to LCD

static void TaskWriteRTCLCD(void *pvParameters); // Write RTC to LCD

static void TaskWriteRTCRetrograde(void *pvParameters); // Write RTC to Retrograde Hands



unsigned char decToBcd(unsigned char val);   // Convert normal decimal numbers to binary coded decimal

unsigned char bcdToDec(unsigned char val);  // Convert binary coded decimal to normal decimal numbers

void setDateDs1307(void);                   // Set the date & time initially & as needed.


/*-----------------------------------------------------------*/

/* structure to pass the DS1307 RTC parameters */
typedef struct
{
    unsigned char    Seconds;       //      
    unsigned char    Minutes;       // 
    unsigned char    Hours;         // 1-12, 0-23 (depending on ampm/24 bit 6)
    unsigned char    AMPM;          //
    unsigned char    Day;           // Sun=1, Mon=2, Tue=3, Wed=4, Thur=5, Fri=6, Sat=7
    unsigned char    Date;          // 
    unsigned char    Month;         // Jan=1,... Dec=12
    unsigned char    Year;          // '00 through '99
    unsigned char    Control;       // 
} xRTCArray, * pRTCArray;


/*-----------------------------------------------------------*/

#endif // CHEADER_RETROGRADE

